from datetime import datetime
import os
import shutil, errno

class FileManager:
    LICENCE = """/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://aheadworks.com/end-user-license-agreement/
 *
 * @package    <PACKAGE_NAME>
 * @version    <VERSION>
 * @copyright  Copyright (c) <COPYRIGHT_YEAR> Aheadworks Inc. (https://aheadworks.com/)
 * @license    https://aheadworks.com/end-user-license-agreement/
 */\n"""

    def remove_files_and_dirs_ignore_case(self, path, files_to_remove, dirs_to_remove):
        for f in os.listdir(path):
            f_path = os.path.join(path, f)
            if os.path.isfile(f_path) and f.lower() in files_to_remove:
                os.remove(f_path)

            if os.path.isdir(f_path) and f.lower() in dirs_to_remove:
                os.system('rm -rf ' + f_path)

    def create_dir_by_path(self, path):
        if not os.path.exists(path):
            os.makedirs(path)

    def create_empty_dir(self, path):
        os.system(f"rm -rf {path} && mkdir -p {path}")

    def find_all(self, name, path):
        """ Recoursively find all files in path by name.
        """
        result = []
        for root, dirs, files in os.walk(path):
            if name in files:
                result.append(os.path.join(root, name))
        return result

    def add_license_to_php_files(self, path, module_name, version):
        # TODO: check if add_info_to_file_header could do this
        license_text = self.get_legal_header(module_name, version)
        for root, dirs, files in os.walk(path):
            for file in files:
                if file.endswith(".php"):
                    with open(os.path.join(root, file)) as f:
                        original_lines = f.readlines()
                    result_lines = list()
                    if len(original_lines):
                        result_lines.append(original_lines[0])
                        result_lines.append(license_text)
                    else:
                        print(f'WARNING: file {os.path.join(root, file)} is empty')
                    for line in original_lines[1:]:
                        result_lines.append(line)
                    result = open(os.path.join(root, file), 'w')
                    result.writelines([item for item in result_lines])
                    result.close()

    def get_legal_header(self, module_name, version, year=None):
        if not year:
            year = str(datetime.now().year)
        license_text = self.LICENCE.replace("<PACKAGE_NAME>", module_name)
        license_text = license_text.replace("<VERSION>", version)
        license_text = license_text.replace("<COPYRIGHT_YEAR>", year)
        return license_text



import re


class Php:

    def __init__(self):
        self.php_vars_by_file_name = {}

    def get_variable_from_file(self, var_name, file):
        file_lines = ''
        for line in open(file):
            file_lines += line

        if file in self.php_vars_by_file_name:
            php_vars = self.php_vars_by_file_name[file]
        else:
            php_vars = re.findall(r"""^define\(\s*['"]*(.*?)['"]*[\s,]+['"]*(.*?)['"]*\s*\)""",
                                  file_lines,
                                  re.IGNORECASE | re.DOTALL | re.MULTILINE)
            php_vars = dict(php_vars)
            self.php_vars_by_file_name[file] = php_vars

        var_value = ''
        if var_name in php_vars:
            var_value = php_vars[var_name]

        return var_value



from aheadworks_core.model.http.api_request import ApiRequest as Api
from jira import JIRA
import json


class JiraApiManager:
    """api manager for jira"""

    def __init__(self, config):
        self.config = config
        self.request = Api(config=self.config)
        self.jira = JIRA(
            server=self.config.url,
            basic_auth=(self.config.user, self.config.token)
        )

    def get_jira_instance(self):
        return self.jira

    def get_issue_url(self, task_key):
        return '{}/browse/{}'.format(self.config.url, task_key)

    def get_release_report_all_issues_url(self, project_key, version_id):
        return '{}/projects/{}/versions/{}/tab/release-report-all-issues'.format(
            self.config.url,
            project_key,
            version_id
        )

    # deprecated, use get_jira_instance
    def search_tasks_jql(self, jql):
        headers = {
            "Accept": "application/json"
        }
        url = '/rest/api/3/search?jql={}'.format(jql)
        data = self.request.request(location=url, headers=headers)

        return json.loads(data)

    # deprecated, use get_jira_instance
    def add_attachments_to_task(self, task_key, files):
        headers = {
            "Accept": "application/json",
            'X-Atlassian-Token': 'nocheck'
        }
        url = '/rest/api/3/issue/{}/attachments'.format(task_key)
        data = self.request.request(location=url, headers=headers, method='POST', files=files)

        return json.loads(data)



from aheadworks_core.model.parser.json import Json as JsonParser


class Console:
    """
    this application needed next env variables
    """

    def __init__(self):
        self.json_parser = JsonParser()

    # Parser
    # first level only. todo improve
    def get_variable_from_json_file(self, var_name, file):
        try:
            print(self.json_parser.get_variable_from_file(var_name, file))
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            exit_code = 1

        exit(exit_code)



import os
from aheadworks_release_manager.api.release_manager import ReleaseManager
from aheadworks_release_manager.api.pipeline_manager import PipelineManager
from aheadworks_core.model.data.jira import JiraConfig
import traceback

class Console:
    """
    this application needed next env variables
    JIRA_USER_EMAIL
    JIRA_TOKEN
    """

    def __init__(self):
        jira_config = JiraConfig(os.getenv('JIRA_USER_EMAIL'), os.getenv('JIRA_TOKEN'))
        self.release_manager = ReleaseManager(jira_config)
        self.pipeline_manager = PipelineManager()

    def teamwork_release(self, jira_project_key, composer_file, discord_bot_url, path_to_files, assign_to):
        try:
            self.release_manager.teamwork_release(
                jira_project_key,
                composer_file,
                discord_bot_url,
                path_to_files,
                assign_to
            )
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            exit_code = 1

        exit(exit_code)

    def jira_release(self, jira_project_key, composer_file, discord_bot_url, path_to_files, assign_to):
        try:
            self.release_manager.jira_release(
                jira_project_key,
                composer_file,
                discord_bot_url,
                path_to_files,
                assign_to
            )
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            exit_code = 1

        exit(exit_code)

    def build_swagger_web_api_doc(
            self,
            path_to_module,
            magento_url,
            magento_path_on_server='/var/www/html'
    ):
        try:
            result = self.release_manager.build_swagger_web_api_doc(
                path_to_module,
                magento_url,
                magento_path_on_server
            )
            print(result)
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1

        exit(exit_code)

    def build_ecommerce_pack(self, bitbucket_workspace, bitbucket_repo_slug):
        try:
            self.release_manager.build_ecommerce_pack(bitbucket_workspace, bitbucket_repo_slug)
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1

        exit(exit_code)

    def build_mm_pack(self):
        try:
            self.release_manager.build_mm_pack()
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1

        exit(exit_code)

    def build_metapackage_pack(self, bitbucket_workspace, bitbucket_repo_slug, artifacts_dir=None):
        try:
            self.release_manager.build_metapackage_pack(bitbucket_workspace, bitbucket_repo_slug, artifacts_dir)
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1

        exit(exit_code)

    def send_pipelines_to_workspace(self, bitbucket_workspace):
        try:
            self.pipeline_manager.send_pipelines_to_workspace(bitbucket_workspace)
            exit_code = 0
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1
        exit(exit_code)



import os
import requests

class TeamworkManager:
    base = ''
    api_base = ''
    token = ''

    def __init__(self, cloud, token):
        self.base = f"https://{cloud}.teamwork.com/"
        self.token = token

    def get_issue_url(self, id):
        return f"{self.base}app/tasks/{id}"

    def find_tasks_by_tags(self, tags):
        str_tags = ",".join(tags)
        url = self.base + f"projects/api/v3/tasks.json?tags={str_tags}&includeCompletedTasks=true"
        response = requests.get(url, auth=(self.token, ''))
        return response.json()['tasks']

    def close_issue(self, task_id):
        url = self.base + f"/tasks/{task_id}.json"
        payload = {
            'todo-item': {
                'completed': True
            }
        }
        response = requests.put(url, auth=(self.token, ''), json=payload)

    def reassign(self, task_id, assign_to):
        url = self.base + f"projects/api/v3/tasks/{task_id}.json"
        payload = {
                "task": {
                    "assignees" :{
                    "userIds" : [assign_to]
                    }
                }
            }
        response = requests.patch(url, auth=(self.token, ''), json=payload)

    def add_comment(self, task_id, text):
        url = self.base + f"tasks/{task_id}/comments.json"
        payload = {
            "comment": {
                "body": text,
                "notify": True
            }
        }
        response = requests.post(url, auth=(self.token, ''), json=payload)

    def add_attachments_to_task(self, task_id, file_names):
        for file_path in file_names:
            # Get a presigned url
            file_name = os.path.basename(file_path)
            file_size = os.stat(file_path)
            url = self.base + f"/projects/api/v1/pendingfiles/presignedurl.json?fileName={file_name}&fileSize={file_size}"
            response = requests.get(url, auth=(self.token, ''))

            ref = response.json()['ref']
            upload_url = response.json()['url']

            # Upload a file to presigned url
            with open(file_path, 'rb') as file:
                files = {'file': (file_path, file)}
                headers = {
                    "X-Amz-Acl": "public-read"
                }
                response = requests.put(
                    upload_url,
                    headers = headers,
                    files=files
                )
        
            # attach the uploaded file to the task
            url = self.base + f"/projects/api/v3/tasks/{task_id}.json"
            payload = {
                "attachments": 
                {
                    "pendingFiles": 
                        [{
                            "reference" :ref 
                        }]
                }
            }
            response = requests.patch(url, auth=(self.token, ''), json=payload)



import subprocess


class SshManager:

    def run_ssh_command(self, command, host, user):
        host = user + '@' + host

        output = subprocess.check_output(
            ["ssh", "%s" % host, command],
            stderr=subprocess.STDOUT,
            universal_newlines=True
        )
        output = output.splitlines()

        return output
